/**
 * noop is a function that does nothing.
 */
export function noop() { }
/**
 * Identity is a function that returns the argument.
 * @param a could be anything
 * @returns a
 */
export function identity(a) {
    return a;
}
/**
 * Always is a function that returns a function that returns the argument.
 * @param a could be anything
 * @returns a function that returns a
 */
export function always(a) {
    return (_) => a;
}
/**
 * assertNever is a function that throws an error if it is ever called.
 * It is used to make sure that a switch statement is exhaustive.
 * @param x
 */
export function assertNever(x) {
    throw new Error(`Unexpected value: ${x}`);
}
export function pipe(...fns) {
    if (!fns) {
        return identity;
    }
    if (fns.length === 1) {
        return fns[0];
    }
    return (t) => {
        // deno-lint-ignore no-explicit-any
        return fns.reduce((prev, fn) => fn(prev), t);
    };
}
/**
 * Pipeline is a class that takes a function and allows you to map over it.
 * The map function returns a new Pipeline.
 * This can be used to compose functions in a more readable way or to compose
 * function in a loop.
 *
 * Example:
 * ```ts
 * const manipulateString = pipeline(upper)
 *   .map(split(''))
 *   .map(reverse)
 *   .map(join).fn;
 * manipulateString('food') // => 'DOOF'
 * ```
 */
export class Pipeline {
    constructor(fn) {
        Object.defineProperty(this, "fn", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: fn
        });
    }
    /**
     * map takes a function and returns a new Pipeline.
     * @param callback
     * @returns
     */
    map(callback) {
        return new Pipeline((a) => {
            const internalValue = this.fn(a);
            return callback(internalValue);
        });
    }
}
/**
 * pipeline is a function that takes a function and returns a Pipeline.
 * @param fn any unary function
 * @returns
 */
export function pipeline(fn) {
    return new Pipeline(fn);
}
export function pick(key, obj) {
    const doit = (obj) => obj[key];
    return typeof obj === 'undefined' ? doit : doit(obj);
}
