"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pick = exports.pipeline = exports.Pipeline = exports.pipe = exports.assertNever = exports.always = exports.identity = exports.noop = void 0;
/**
 * noop is a function that does nothing.
 */
function noop() { }
exports.noop = noop;
/**
 * Identity is a function that returns the argument.
 * @param a could be anything
 * @returns a
 */
function identity(a) {
    return a;
}
exports.identity = identity;
/**
 * Always is a function that returns a function that returns the argument.
 * @param a could be anything
 * @returns a function that returns a
 */
function always(a) {
    return (_) => a;
}
exports.always = always;
/**
 * assertNever is a function that throws an error if it is ever called.
 * It is used to make sure that a switch statement is exhaustive.
 * @param x
 */
function assertNever(x) {
    throw new Error(`Unexpected value: ${x}`);
}
exports.assertNever = assertNever;
function pipe(...fns) {
    if (!fns) {
        return identity;
    }
    if (fns.length === 1) {
        return fns[0];
    }
    return (t) => {
        // deno-lint-ignore no-explicit-any
        return fns.reduce((prev, fn) => fn(prev), t);
    };
}
exports.pipe = pipe;
/**
 * Pipeline is a class that takes a function and allows you to map over it.
 * The map function returns a new Pipeline.
 * This can be used to compose functions in a more readable way or to compose
 * function in a loop.
 *
 * Example:
 * ```ts
 * const manipulateString = pipeline(upper)
 *   .map(split(''))
 *   .map(reverse)
 *   .map(join).fn;
 * manipulateString('food') // => 'DOOF'
 * ```
 */
class Pipeline {
    constructor(fn) {
        Object.defineProperty(this, "fn", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: fn
        });
    }
    /**
     * map takes a function and returns a new Pipeline.
     * @param callback
     * @returns
     */
    map(callback) {
        return new Pipeline((a) => {
            const internalValue = this.fn(a);
            return callback(internalValue);
        });
    }
}
exports.Pipeline = Pipeline;
/**
 * pipeline is a function that takes a function and returns a Pipeline.
 * @param fn any unary function
 * @returns
 */
function pipeline(fn) {
    return new Pipeline(fn);
}
exports.pipeline = pipeline;
function pick(key, obj) {
    const doit = (obj) => obj[key];
    return typeof obj === 'undefined' ? doit : doit(obj);
}
exports.pick = pick;
